@@ -83,7 +83,7 @@ AdBlockRule::AdBlockRule(const QString &filter, AdBlockSubscription* subscriptio
     , m_isEnabled(true)
     , m_isException(false)
     , m_isInternalDisabled(false)
-    , m_regExp(0)
+    , m_regExp(nullptr)
 {
     setFilter(filter);
 }
@@ -149,6 +149,11 @@ QString AdBlockRule::cssSelector() const
     return m_matchString;
 }
 
+bool AdBlockRule::isUnsupportedRule() const
+{
+    return m_type == ExtendedCssRule || m_type == SnippetRule || m_isInternalDisabled;
+}
+
 bool AdBlockRule::isDocument() const
 {
     return hasOption(DocumentOption);
@@ -159,6 +164,11 @@ bool AdBlockRule::isElemhide() const
     return hasOption(ElementHideOption);
 }
 
+bool AdBlockRule::isGenerichide() const
+{
+    return hasOption(GenericHideOption);
+}
+
 bool AdBlockRule::isDomainRestricted() const
 {
     return hasOption(DomainRestrictedOption);
@@ -186,7 +196,7 @@ void AdBlockRule::setEnabled(bool enabled)
 
 bool AdBlockRule::isSlow() const
 {
-    return m_regExp != 0;
+    return m_regExp != nullptr;
 }
 
 bool AdBlockRule::isInternalDisabled() const
@@ -196,7 +206,7 @@ bool AdBlockRule::isInternalDisabled() const
 
 bool AdBlockRule::urlMatch(const QUrl &url) const
 {
-    if (!hasOption(DocumentOption) && !hasOption(ElementHideOption)) {
+    if (!hasOption(DocumentOption) && !hasOption(ElementHideOption) && !hasOption(GenericHideOption) && !hasOption(GenericBlockOption)) {
         return false;
     }
 
@@ -225,55 +235,9 @@ bool AdBlockRule::networkMatch(const QWebEngineUrlRequestInfo &request, const QS
             return false;
         }
 
-        // Check object restrictions
-        if (hasOption(ObjectOption) && !matchObject(request)) {
+        // Check type restrictions
+        if (((m_exceptions | m_options) & TypeOptions) && !matchType(request))
             return false;
-        }
-
-        // Check subdocument restriction
-        if (hasOption(SubdocumentOption) && !matchSubdocument(request)) {
-            return false;
-        }
-
-        // Check xmlhttprequest restriction
-        if (hasOption(XMLHttpRequestOption) && !matchXmlHttpRequest(request)) {
-            return false;
-        }
-
-        // Check image restriction
-        if (hasOption(ImageOption) && !matchImage(request)) {
-            return false;
-        }
-
-        // Check script restriction
-        if (hasOption(ScriptOption) && !matchScript(request)) {
-            return false;
-        }
-
-        // Check stylesheet restriction
-        if (hasOption(StyleSheetOption) && !matchStyleSheet(request)) {
-            return false;
-        }
-
-        // Check object-subrequest restriction
-        if (hasOption(ObjectSubrequestOption) && !matchObjectSubrequest(request)) {
-            return false;
-        }
-
-        // Check ping restriction
-        if (hasOption(PingOption) && !matchPing(request)) {
-            return false;
-        }
-
-        // Check media restriction
-        if (hasOption(MediaOption) && !matchMedia(request)) {
-            return false;
-        }
-
-        // Check font restriction
-        if (hasOption(FontOption) && !matchFont(request)) {
-            return false;
-        }
     }
 
     return matched;
@@ -290,14 +254,14 @@ bool AdBlockRule::matchDomain(const QString &domain) const
     }
 
     if (m_blockedDomains.isEmpty()) {
-        foreach (const QString &d, m_allowedDomains) {
+        for (const QString &d : qAsConst(m_allowedDomains)) {
             if (isMatchingDomain(domain, d)) {
                 return true;
             }
         }
     }
     else if (m_allowedDomains.isEmpty()) {
-        foreach (const QString &d, m_blockedDomains) {
+        for (const QString &d : qAsConst(m_blockedDomains)) {
             if (isMatchingDomain(domain, d)) {
                 return false;
             }
@@ -305,13 +269,13 @@ bool AdBlockRule::matchDomain(const QString &domain) const
         return true;
     }
     else {
-        foreach (const QString &d, m_blockedDomains) {
+        for (const QString &d : qAsConst(m_blockedDomains)) {
             if (isMatchingDomain(domain, d)) {
                 return false;
             }
         }
 
-        foreach (const QString &d, m_allowedDomains) {
+        for (const QString &d : qAsConst(m_allowedDomains)) {
             if (isMatchingDomain(domain, d)) {
                 return true;
             }
@@ -332,88 +296,60 @@ bool AdBlockRule::matchThirdParty(const QWebEngineUrlRequestInfo &request) const
     return hasException(ThirdPartyOption) ? !match : match;
 }
 
-bool AdBlockRule::matchObject(const QWebEngineUrlRequestInfo &request) const
+bool AdBlockRule::matchType(const QWebEngineUrlRequestInfo &request) const
 {
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeObject;
-
-    return hasException(ObjectOption) ? !match : match;
-}
-
-bool AdBlockRule::matchSubdocument(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeSubFrame;
-
-    return hasException(SubdocumentOption) ? !match : match;
-}
-
-bool AdBlockRule::matchXmlHttpRequest(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeXhr;
-
-    return hasException(XMLHttpRequestOption) ? !match : match;
-}
-
-bool AdBlockRule::matchImage(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeImage;
-
-    return hasException(ImageOption) ? !match : match;
-}
-
-bool AdBlockRule::matchScript(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeScript;
-
-    return hasException(ScriptOption) ? !match : match;
-}
-
-bool AdBlockRule::matchStyleSheet(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeStylesheet;
-
-    return hasException(StyleSheetOption) ? !match : match;
-}
-
-bool AdBlockRule::matchObjectSubrequest(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypePluginResource;
-
-    return hasException(ObjectSubrequestOption) ? !match : match;
-}
-
-bool AdBlockRule::matchPing(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypePing;
-
-    return hasException(PingOption) ? !match : match;
-}
-
-bool AdBlockRule::matchMedia(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeMedia;
-
-    return hasException(MediaOption) ? !match : match;
-}
-
-bool AdBlockRule::matchFont(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeFontResource;
-
-    return hasException(FontOption) ? !match : match;
-}
-
-bool AdBlockRule::matchOther(const QWebEngineUrlRequestInfo &request) const
-{
-    bool match = request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeFontResource
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeSubResource
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeWorker
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeSharedWorker
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypePrefetch
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeFavicon
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeServiceWorker
-              || request.resourceType() == QWebEngineUrlRequestInfo::ResourceTypeUnknown;
-
-    return hasException(MediaOption) ? !match : match;
+    RuleOption type;
+    switch (request.resourceType()) {
+    case QWebEngineUrlRequestInfo::ResourceTypeMainFrame:
+        type = DocumentOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeSubFrame:
+        type = SubdocumentOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeStylesheet:
+        type = StyleSheetOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeScript:
+        type = ScriptOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeImage:
+        type = ImageOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeFontResource:
+        type = FontOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeObject:
+        type = ObjectOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeMedia:
+        type = MediaOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeXhr:
+        type = XMLHttpRequestOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypePing:
+        type = PingOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypePluginResource:
+        type = ObjectSubrequestOption;
+        break;
+    case QWebEngineUrlRequestInfo::ResourceTypeSubResource:
+    case QWebEngineUrlRequestInfo::ResourceTypeWorker:
+    case QWebEngineUrlRequestInfo::ResourceTypeSharedWorker:
+    case QWebEngineUrlRequestInfo::ResourceTypePrefetch:
+    case QWebEngineUrlRequestInfo::ResourceTypeFavicon:
+    case QWebEngineUrlRequestInfo::ResourceTypeServiceWorker:
+    case QWebEngineUrlRequestInfo::ResourceTypeCspReport:
+    case QWebEngineUrlRequestInfo::ResourceTypeNavigationPreloadMainFrame:
+    case QWebEngineUrlRequestInfo::ResourceTypeNavigationPreloadSubFrame:
+    case QWebEngineUrlRequestInfo::ResourceTypeUnknown:
+    default:
+        type = OtherOption;
+        break;
+    }
+    if (!m_exceptions)
+        return m_options.testFlag(type);
+    return !m_exceptions.testFlag(type);
 }
 
 void AdBlockRule::parseFilter()
@@ -430,13 +366,44 @@ void AdBlockRule::parseFilter()
         return;
     }
 
+    // Exception always starts with @@
+    if (parsedLine.startsWith(QL1S("@@"))) {
+        m_isException = true;
+        parsedLine.remove(0, 2);
+    }
+
+    // Extended CSS element hiding
+    if (parsedLine.contains(QL1S("#?#"))) {
+        m_type = ExtendedCssRule;
+        int pos = parsedLine.indexOf(QL1C('#'));
+        if (!parsedLine.startsWith(QL1S("#"))) {
+            QString domains = parsedLine.left(pos);
+            parseDomains(domains, QL1C(','));
+        }
+        m_matchString = parsedLine.mid(pos + 3);
+        // CSS rule cannot have more options -> stop parsing
+        return;
+    }
+
+    // Snippet rule
+    if (parsedLine.contains(QL1S("#$#"))) {
+        m_type = SnippetRule;
+        int pos = parsedLine.indexOf(QL1C('#'));
+        if (!parsedLine.startsWith(QL1S("#"))) {
+            QString domains = parsedLine.left(pos);
+            parseDomains(domains, QL1C(','));
+        }
+        m_matchString = parsedLine.mid(pos + 3);
+        return;
+    }
+
     // CSS Element hiding rule
     if (parsedLine.contains(QL1S("##")) || parsedLine.contains(QL1S("#@#"))) {
         m_type = CssRule;
         int pos = parsedLine.indexOf(QL1C('#'));
 
         // Domain restricted rule
-        if (!parsedLine.startsWith(QL1S("##"))) {
+        if (!parsedLine.startsWith(QL1S("#"))) {
             QString domains = parsedLine.left(pos);
             parseDomains(domains, QL1C(','));
         }
@@ -448,19 +415,13 @@ void AdBlockRule::parseFilter()
         return;
     }
 
-    // Exception always starts with @@
-    if (parsedLine.startsWith(QL1S("@@"))) {
-        m_isException = true;
-        parsedLine = parsedLine.mid(2);
-    }
-
     // Parse all options following $ char
     int optionsIndex = parsedLine.indexOf(QL1C('$'));
     if (optionsIndex >= 0) {
         const QStringList options = parsedLine.mid(optionsIndex + 1).split(QL1C(','), QString::SkipEmptyParts);
 
         int handledOptions = 0;
-        foreach (const QString &option, options) {
+        for (const QString &option : options) {
             if (option.startsWith(QL1S("domain="))) {
                 parseDomains(option.mid(7), QL1C('|'));
                 ++handledOptions;
@@ -529,6 +490,15 @@ void AdBlockRule::parseFilter()
                 setException(OtherOption, option.startsWith(QL1C('~')));
                 ++handledOptions;
             }
+            else if (option == QL1S("collapse")) {
+                // Hiding placeholders of blocked elements is enabled by default
+                ++handledOptions;
+            }
+            else if (option == QL1S("popup")) {
+                // doesn't do anything yet
+                setOption(PopupOption);
+                ++handledOptions;
+            }
             else if (option == QL1S("document") && m_isException) {
                 setOption(DocumentOption);
                 ++handledOptions;
@@ -537,10 +507,15 @@ void AdBlockRule::parseFilter()
                 setOption(ElementHideOption);
                 ++handledOptions;
             }
-            else if (option == QL1S("collapse")) {
-                // Hiding placeholders of blocked elements is enabled by default
+            else if (option == QL1S("generichide") && m_isException) {
+                setOption(GenericHideOption);
                 ++handledOptions;
             }
+            else if (option == QL1S("genericblock") && m_isException) {
+                // doesn't do anything yet
+                setOption(GenericBlockOption);
+//                 ++handledOptions;
+            }
         }
 
         // If we don't handle all options, it's safer to just disable this rule
@@ -550,12 +525,12 @@ void AdBlockRule::parseFilter()
             return;
         }
 
-        parsedLine = parsedLine.left(optionsIndex);
+        parsedLine.truncate(optionsIndex);
     }
 
     // Rule is classic regexp
     if (parsedLine.startsWith(QL1C('/')) && parsedLine.endsWith(QL1C('/'))) {
-        parsedLine = parsedLine.mid(1);
+        parsedLine.remove(0, 1);
         parsedLine = parsedLine.left(parsedLine.size() - 1);
 
         m_type = RegExpMatchRule;
@@ -570,7 +545,7 @@ void AdBlockRule::parseFilter()
 
     // Remove starting and ending wildcards (*)
     if (parsedLine.startsWith(QL1C('*'))) {
-        parsedLine = parsedLine.mid(1);
+        parsedLine.remove(0, 1);
     }
 
     if (parsedLine.endsWith(QL1C('*'))) {
@@ -579,7 +554,7 @@ void AdBlockRule::parseFilter()
 
     // We can use fast string matching for domain here
     if (filterIsOnlyDomain(parsedLine)) {
-        parsedLine = parsedLine.mid(2);
+        parsedLine.remove(0, 2);
         parsedLine = parsedLine.left(parsedLine.size() - 1);
 
         m_type = DomainMatchRule;
@@ -631,9 +606,9 @@ void AdBlockRule::parseFilter()
 
 void AdBlockRule::parseDomains(const QString &domains, const QChar &separator)
 {
-    QStringList domainsList = domains.split(separator, QString::SkipEmptyParts);
+    const QStringList domainsList = domains.split(separator, QString::SkipEmptyParts);
 
-    foreach (const QString domain, domainsList) {
+    for (const QString domain : domainsList) {
         if (domain.isEmpty()) {
             continue;
         }
@@ -720,7 +695,7 @@ QString AdBlockRule::createRegExpFromFilter(const QString &filter) const
                     i++;
                 }
                 else {
-                    parsed.append('^');
+                    parsed.append(QL1C('^'));
                 }
                 break;
             }
@@ -748,7 +723,7 @@ QList<QStringMatcher> AdBlockRule::createStringMatchers(const QStringList &filte
     QList<QStringMatcher> matchers;
     matchers.reserve(filters.size());
 
-    foreach (const QString &filter, filters) {
+    for (const QString &filter : filters) {
         matchers.append(QStringMatcher(filter, m_caseSensitivity));
     }
 
@@ -790,7 +765,8 @@ bool AdBlockRule::isMatchingRegExpStrings(const QString &url) const
 {
     Q_ASSERT(m_regExp);
 
-    foreach (const QStringMatcher &matcher, m_regExp->matchers) {
+    const auto matchers = m_regExp->matchers;
+    for (const QStringMatcher &matcher : matchers) {
         if (matcher.indexIn(url) == -1)
             return false;
     }
