@@ -47,6 +47,7 @@ private Q_SLOTS:
     void openBookmarkInNewTab();
     void openBookmarkInNewWindow();
     void openBookmarkInNewPrivateWindow();
+    void createNewFolder();
     void editBookmark();
     void deleteBookmark();
 
@@ -57,8 +58,11 @@ private:
 
     QSize minimumSizeHint() const override;
 
-    void dropEvent(QDropEvent* e);
-    void dragEnterEvent(QDragEnterEvent* e);
+    void dropEvent(QDropEvent* e) override;
+    void dragEnterEvent(QDragEnterEvent* e) override;
+    void dragMoveEvent(QDragMoveEvent* e) override;
+    void dragLeaveEvent(QDragLeaveEvent* e) override;
+    void paintEvent(QPaintEvent* p) override;
 
     BrowserWindow* m_window;
     Bookmarks* m_bookmarks;
@@ -67,6 +71,10 @@ private:
     QTimer* m_updateTimer;
     QAction* m_actShowOnlyIcons = nullptr;
     QAction* m_actShowOnlyText = nullptr;
+
+    int m_dropRow;
+    QPoint m_dropPos;
+    void clearDropIndicator();
 };
 
 #endif // BOOKMARKSTOOLBAR_H
